clear all;
close all;

directory_name = 'cones'; % 'cones' or 'teddy'
noise_type = 'gaussian'; % uniform or gaussian
viewpos = 0.5; % 0.5 or 0.25

global interp_type; interp_type = 'spline';
margin_size = 50;
disparity_constant = 4;

% read input images

disp('read input images...');
switch viewpos
    case 0.25
        left_image = double(imread(sprintf('%s/im1.pgm', directory_name)));
        right_image = double(imread(sprintf('%s/im5.pgm', directory_name)));        
    case 0.5
        left_image = double(imread(sprintf('%s/im0.pgm', directory_name)));
        right_image = double(imread(sprintf('%s/im4.pgm', directory_name)));
    otherwise
        disp('viewpos should be 0.5 or 0.25');
        return;
end

center_image = double(imread(sprintf('%s/im2.pgm', directory_name)));
depth_image = double(imread(sprintf('%s/disp2.pgm', directory_name)))/(disparity_constant);

% set void disparity to 50 pixels from both sides

depth_image(:,1:margin_size) = 0;
depth_image(:,end-margin_size+1:end) = 0;


% construct noise array

if strcmp(noise_type, 'gaussian')
    noise_array = [0.0, 0.5, 1.0, 1.5, 2.0, 2.5, 3.0, 3.5, 4.0, 4.5, 5.0];
elseif strcmp(noise_type, 'uniform')
    noise_array = [0.0, 0.5, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.5];
else
    disp('noise_type should be gaussian or uniform');
    return;
end

MSE1_array = zeros(1, length(noise_array));
MSE2_array = zeros(1, length(noise_array));
MSE3_array = zeros(1, length(noise_array));



for index = 1:length(noise_array);
    
    noise_param = noise_array(index);
    
    % generate noised depth map
    
    fprintf(1, 'add noise (%f) to the depth image .....\n', noise_param);
    
    if strcmp(noise_type,'gaussian')
        noise_image = randn(size(center_image)) .* (depth_image ~= 0);
    elseif strcmp(noise_type, 'uniform')
        noise_image = 2* (rand(size(center_image))- 0.5) .* (depth_image ~= 0);
    else
        noise_image = 0;
    end
    depth_image_n = depth_image + noise_param * noise_image;
    
    % perform view interpolation with uniform noise
    
    disp('view interpolation: linear');
    
    result_image1 = ViewInterpolation(left_image, right_image, depth_image_n, viewpos,...
        noise_param, noise_type, 'none');
    [MSE1, PSNR1, residual_image1] = CompareImage(result_image1, center_image, depth_image);
    disp(['  MSE = ', num2str(MSE1), ' PSNR = ', num2str(PSNR1)]);
    
    
    disp('view interpolation: optimal');
    
    result_image2 = ViewInterpolation(left_image, right_image, depth_image_n, viewpos,...
        noise_param, noise_type, 'prefilter');
    [MSE2, PSNR2, residual_image2] = CompareImage(result_image2, center_image, depth_image);
    disp(['  MSE = ', num2str(MSE2), ' PSNR = ', num2str(PSNR2)]);
    
    disp('view interplation: band-limited linear');
    result_image3 = ViewInterpolation(...
        Bandlimit(left_image, viewpos, noise_type, noise_param), ...
        Bandlimit(right_image, viewpos, noise_type, noise_param), ...
        depth_image_n, viewpos,...
        noise_param, noise_type, 'none');
    [MSE3, PSNR3, residual_image3] = CompareImage(result_image3, center_image, depth_image);
    disp(['  MSE = ', num2str(MSE3), ' PSNR = ', num2str(PSNR3)]);

    
    MSE1_array(index) = MSE1;
    MSE2_array(index) = MSE2;
    MSE3_array(index) = MSE3;
    
    % save images
    
    image_filename = sprintf('results/%s_viewpos=%.3d_noisetype=%s_noiselevel=%.2d_interpmethod=linear.pgm', ...
        directory_name, viewpos*100, noise_type, index);
    imwrite(uint8(result_image1), image_filename);
    image_filename = sprintf('results/%s_viewpos=%.3d_noisetype=%s_noiselevel=%.2d_interpmethod=optimal.pgm', ...
        directory_name, viewpos*100, noise_type, index);
    imwrite(uint8(result_image2), image_filename);
    image_filename = sprintf('results/%s_viewpos=%.3d_noisetype=%s_noiselevel=%.2d_interpmethod=band-limited.pgm', ...
        directory_name, viewpos*100, noise_type, index);
    imwrite(uint8(result_image3), image_filename);    
    
end

% draw graphcs

h = figure;
plot(noise_array, MSE1_array, noise_array, MSE2_array, noise_array, MSE3_array);
legend('linear', 'optimal', 'band-limited');

if strcmp(directory_name, 'teddy')
    max_MSE = 250;
else
    max_MSE = 350;
end

axis([0, max(noise_array), 0, max_MSE]);
graph_filename = sprintf('results/%s_%s_%.3d_graph.fig', ...
            directory_name, noise_type, viewpos*100);
saveas(h, graph_filename);

array_filename = sprintf('results/%s_%s_%.3d.mat', ...
            directory_name, noise_type, viewpos*100);
save(array_filename, 'noise_array', 'MSE1_array', 'MSE2_array', 'MSE3_array', '-ASCII');
